@@ -1,4 +1,7 @@
 #!/bin/bash
+# The following is to disable "info" warnings for the unquoted instandes of $MESON_FLAGS
+# in the meson invocations below.  We want the shell to actually split $MESON_FLAGS by spaces.
+# shellcheck disable=SC2086
 #
 # IMPORTANT: See
 # https://gnome.pages.gitlab.gnome.org/librsvg/devel-docs/ci.html#container-image-version
@@ -18,11 +21,6 @@ LIBXML2_TAG="v2.13.3"
 GDK_PIXBUF_TAG="2.42.12"
 
 PARSED=$(getopt --options '' --longoptions 'prefix:,meson-flags:' --name "$0" -- "$@")
-if [ $? -ne 0 ]; then
-	echo 'Terminating...' >&2
-	exit 1
-fi
-
 eval set -- "$PARSED"
 unset PARSED
 
@@ -64,35 +62,35 @@ source ci/setup-dependencies-env.sh
 cd ..
 git clone --depth 1 --branch $FREETYPE2_TAG https://gitlab.freedesktop.org/freetype/freetype
 cd freetype
-meson setup _build --prefix $PREFIX -Dharfbuzz=disabled $MESON_FLAGS
+meson setup _build --prefix "$PREFIX" -Dharfbuzz=disabled $MESON_FLAGS
 meson compile -C _build
 meson install -C _build
 
 cd ..
 git clone --depth 1 --branch $FONTCONFIG_TAG https://gitlab.freedesktop.org/fontconfig/fontconfig
 cd fontconfig
-meson setup _build --prefix $PREFIX $MESON_FLAGS
+meson setup _build --prefix "$PREFIX" $MESON_FLAGS
 meson compile -C _build
 meson install -C _build
 
 cd ..
 git clone --depth 1 --branch $CAIRO_TAG https://gitlab.freedesktop.org/cairo/cairo
 cd cairo
-meson setup _build --prefix $PREFIX $MESON_FLAGS
+meson setup _build --prefix "$PREFIX" $MESON_FLAGS
 meson compile -C _build
 meson install -C _build
 
 cd ..
 git clone --depth 1 --branch $HARFBUZZ_TAG https://github.com/harfbuzz/harfbuzz
 cd harfbuzz
-meson setup _build --prefix $PREFIX $MESON_FLAGS
+meson setup _build --prefix "$PREFIX" $MESON_FLAGS
 meson compile -C _build
 meson install -C _build
 
 cd ..
 git clone --depth 1 --branch $PANGO_TAG https://gitlab.gnome.org/GNOME/pango
 cd pango
-meson setup _build --prefix $PREFIX $MESON_FLAGS
+meson setup _build --prefix "$PREFIX" $MESON_FLAGS
 meson compile -C _build
 meson install -C _build
 
@@ -101,13 +99,13 @@ git clone --depth 1 --branch $LIBXML2_TAG https://gitlab.gnome.org/GNOME/libxml2
 cd libxml2
 mkdir _build
 cd _build
-../autogen.sh --prefix $PREFIX --libdir $PREFIX/lib64 --without-python
+../autogen.sh --prefix "$PREFIX" --libdir "$PREFIX"/lib64 --without-python
 make
 make install
 
 cd ..
 git clone --depth 1 --branch $GDK_PIXBUF_TAG https://gitlab.gnome.org/GNOME/gdk-pixbuf
 cd gdk-pixbuf
-meson setup _build --prefix $PREFIX -Dman=false $MESON_FLAGS
+meson setup _build --prefix "$PREFIX" -Dman=false $MESON_FLAGS
 meson compile -C _build
 meson install -C _build
