@@ -18,7 +18,7 @@ pub use crate::{
 use crate::{
     accept_language::{LanguageTags, UserLanguage},
     css::{Origin, Stylesheet},
-    document::{Document, LoadOptions, NodeId},
+    document::{Document, LoadOptions, NodeId, RenderingOptions},
     dpi::Dpi,
     drawing_ctx::SvgNesting,
     error::InternalRenderingError,
@@ -57,6 +57,11 @@ pub enum RenderingError {
 
     /// Not enough memory was available for rendering.
     OutOfMemory(String),
+
+    /// The rendering was interrupted via a [`gio::Cancellable`].
+    ///
+    /// See the documentation for [`CairoRenderer::with_cancellable`].
+    Cancelled,
 }
 
 impl std::error::Error for RenderingError {}
@@ -79,9 +84,13 @@ impl From<InternalRenderingError> for RenderingError {
             InternalRenderingError::InvalidTransform => {
                 RenderingError::Rendering("invalid transform".to_string())
             }
+            InternalRenderingError::CircularReference(c) => {
+                RenderingError::Rendering(format!("circular reference in node {c}"))
+            }
             InternalRenderingError::IdNotFound => RenderingError::IdNotFound,
             InternalRenderingError::InvalidId(s) => RenderingError::InvalidId(s),
             InternalRenderingError::OutOfMemory(s) => RenderingError::OutOfMemory(s),
+            InternalRenderingError::Cancelled => RenderingError::Cancelled,
         }
     }
 }
@@ -94,6 +103,7 @@ impl fmt::Display for RenderingError {
             RenderingError::IdNotFound => write!(f, "element id not found"),
             RenderingError::InvalidId(ref s) => write!(f, "invalid id: {s:?}"),
             RenderingError::OutOfMemory(ref s) => write!(f, "out of memory: {s}"),
+            RenderingError::Cancelled => write!(f, "rendering cancelled"),
         }
     }
 }
@@ -116,12 +126,12 @@ impl Loader {
     /// Creates a `Loader` with the default flags.
     ///
     /// * [`unlimited_size`](#method.with_unlimited_size) defaults to `false`, as malicious
-    /// SVG documents could cause the XML parser to consume very large amounts of memory.
+    ///   SVG documents could cause the XML parser to consume very large amounts of memory.
     ///
     /// * [`keep_image_data`](#method.keep_image_data) defaults to
-    /// `false`.  You may only need this if rendering to Cairo
-    /// surfaces that support including image data in compressed
-    /// formats, like PDF.
+    ///   `false`.  You may only need this if rendering to Cairo
+    ///   surfaces that support including image data in compressed
+    ///   formats, like PDF.
     ///
     /// # Example:
     ///
@@ -352,7 +362,7 @@ impl SvgHandle {
             Origin::User,
             self.session.clone(),
         )?;
-        self.document.cascade(&[stylesheet], &self.session);
+        self.document.cascade(&[stylesheet]);
         Ok(())
     }
 }
@@ -427,6 +437,7 @@ pub struct CairoRenderer<'a> {
     pub(crate) handle: &'a SvgHandle,
     pub(crate) dpi: Dpi,
     user_language: UserLanguage,
+    cancellable: Option<gio::Cancellable>,
     is_testing: bool,
 }
 
@@ -521,6 +532,7 @@ impl<'a> CairoRenderer<'a> {
             handle,
             dpi: Dpi::new(DEFAULT_DPI_X, DEFAULT_DPI_Y),
             user_language: UserLanguage::new(&Language::FromEnvironment, session),
+            cancellable: None,
             is_testing: false,
         }
     }
@@ -559,6 +571,30 @@ impl<'a> CairoRenderer<'a> {
         }
     }
 
+    /// Sets a cancellable to be able to interrupt rendering.
+    ///
+    /// The rendering functions like [`render_document`] will normally render the whole
+    /// SVG document tree.  However, they can be interrupted if you set a `cancellable`
+    /// object with this method.  To interrupt rendering, you can call
+    /// [`gio::CancellableExt::cancel()`] from a different thread than where the rendering
+    /// is happening.
+    ///
+    /// Since rendering happens as a side-effect on the Cairo context (`cr`) that is
+    /// passed to the rendering functions, it may be that the `cr`'s target surface is in
+    /// an undefined state if the rendering is cancelled.  The surface may have not yet
+    /// been painted on, or it may contain a partially-rendered document.  For this
+    /// reason, if your application does not want to leave the target surface in an
+    /// inconsistent state, you may prefer to use a temporary surface for rendering, which
+    /// can be discarded if your code cancels the rendering.
+    ///
+    /// [`render_document`]: #method.render_document
+    pub fn with_cancellable<C: IsA<Cancellable>>(self, cancellable: &C) -> Self {
+        CairoRenderer {
+            cancellable: Some(cancellable.clone().into()),
+            ..self
+        }
+    }
+
     /// Queries the `width`, `height`, and `viewBox` attributes in an SVG document.
     ///
     /// If you are calling this function to compute a scaling factor to render the SVG,
@@ -607,6 +643,16 @@ impl<'a> CairoRenderer<'a> {
         Some(self.width_height_to_user(self.dpi))
     }
 
+    fn rendering_options(&self) -> RenderingOptions {
+        RenderingOptions {
+            dpi: self.dpi,
+            cancellable: self.cancellable.clone(),
+            user_language: self.user_language.clone(),
+            svg_nesting: SvgNesting::Standalone,
+            testing: self.is_testing,
+        }
+    }
+
     /// Renders the whole SVG document fitted to a viewport
     ///
     /// The `viewport` gives the position and size at which the whole SVG
@@ -620,15 +666,10 @@ impl<'a> CairoRenderer<'a> {
         cr: &cairo::Context,
         viewport: &cairo::Rectangle,
     ) -> Result<(), RenderingError> {
-        Ok(self.handle.document.render_document(
-            &self.handle.session,
-            cr,
-            viewport,
-            &self.user_language,
-            self.dpi,
-            SvgNesting::Standalone,
-            self.is_testing,
-        )?)
+        Ok(self
+            .handle
+            .document
+            .render_document(cr, viewport, &self.rendering_options())?)
     }
 
     /// Computes the (ink_rect, logical_rect) of an SVG element, as if
@@ -664,12 +705,9 @@ impl<'a> CairoRenderer<'a> {
         let node = self.handle.get_node_or_root(&node_id)?;
 
         Ok(self.handle.document.get_geometry_for_layer(
-            &self.handle.session,
             node,
             viewport,
-            &self.user_language,
-            self.dpi,
-            self.is_testing,
+            &self.rendering_options(),
         )?)
     }
 
@@ -701,16 +739,10 @@ impl<'a> CairoRenderer<'a> {
         let node_id = self.handle.get_node_id_or_root(id)?;
         let node = self.handle.get_node_or_root(&node_id)?;
 
-        Ok(self.handle.document.render_layer(
-            &self.handle.session,
-            cr,
-            node,
-            viewport,
-            &self.user_language,
-            self.dpi,
-            SvgNesting::Standalone,
-            self.is_testing,
-        )?)
+        Ok(self
+            .handle
+            .document
+            .render_layer(cr, node, viewport, &self.rendering_options())?)
     }
 
     /// Computes the (ink_rect, logical_rect) of a single SVG element
@@ -750,13 +782,10 @@ impl<'a> CairoRenderer<'a> {
         let node_id = self.handle.get_node_id_or_root(id)?;
         let node = self.handle.get_node_or_root(&node_id)?;
 
-        Ok(self.handle.document.get_geometry_for_element(
-            &self.handle.session,
-            node,
-            &self.user_language,
-            self.dpi,
-            self.is_testing,
-        )?)
+        Ok(self
+            .handle
+            .document
+            .get_geometry_for_element(node, &self.rendering_options())?)
     }
 
     /// Renders a single SVG element to a given viewport
@@ -785,13 +814,10 @@ impl<'a> CairoRenderer<'a> {
         let node = self.handle.get_node_or_root(&node_id)?;
 
         Ok(self.handle.document.render_element(
-            &self.handle.session,
             cr,
             node,
             element_viewport,
-            &self.user_language,
-            self.dpi,
-            self.is_testing,
+            &self.rendering_options(),
         )?)
     }
 
@@ -813,12 +839,12 @@ impl<'a> CairoRenderer<'a> {
         let width = dimensions.width;
         let height = dimensions.height;
 
-        let view_params = Viewport::new(dpi, 0.0, 0.0);
+        let viewport = Viewport::new(dpi, 0.0, 0.0);
         let root = self.handle.document.root();
         let cascaded = CascadedValues::new_from_node(&root);
         let values = cascaded.get();
 
-        let params = NormalizeParams::new(values, &view_params);
+        let params = NormalizeParams::new(values, &viewport);
 
         (width.to_user(&params), height.to_user(&params))
     }
