@@ -7,24 +7,26 @@ use gio::{Cancellable, FileCreateFlags, InputStream, OutputStream};
 #[cfg(unix)]
 use gio::{UnixInputStream, UnixOutputStream};
 
+use glib::translate::*;
+
 #[cfg(windows)]
 mod windows_imports {
     pub use gio::{Win32InputStream, WriteOutputStream};
-    pub use glib::translate::*;
 }
 #[cfg(windows)]
 use self::windows_imports::*;
 
-use cssparser::{match_ignore_ascii_case, Color};
+use cssparser::match_ignore_ascii_case;
 
 use librsvg_c::{handle::PathOrUrl, sizing::LegacySize};
 use rsvg::rsvg_convert_only::{
-    set_source_color_on_cairo, AspectRatio, CssLength, Dpi, Horizontal, Length, Normalize,
-    NormalizeParams, Parse, Rect, Signed, ULength, Unsigned, Validate, Vertical, ViewBox,
+    set_source_color_on_cairo, AspectRatio, Color, CssLength, Dpi, Horizontal, Length, Normalize,
+    NormalizeParams, Parse, Rect, ULength, Validate, Vertical, ViewBox,
 };
 use rsvg::{AcceptLanguage, CairoRenderer, Language, LengthUnit, Loader, RenderingError};
 
-use std::ffi::OsString;
+use rsvg_convert::*;
+
 use std::io;
 use std::io::IsTerminal;
 use std::ops::Deref;
@@ -802,271 +804,67 @@ fn natural_geometry(
     })
 }
 
-fn build_cli() -> clap::Command {
-    let supported_formats = vec![
-        "png",
-        #[cfg(system_deps_have_cairo_pdf)]
-        "pdf",
-        #[cfg(system_deps_have_cairo_pdf)]
-        "pdf1.7",
-        #[cfg(system_deps_have_cairo_pdf)]
-        "pdf1.6",
-        #[cfg(system_deps_have_cairo_pdf)]
-        "pdf1.5",
-        #[cfg(system_deps_have_cairo_pdf)]
-        "pdf1.4",
-        #[cfg(system_deps_have_cairo_ps)]
-        "ps",
-        #[cfg(system_deps_have_cairo_ps)]
-        "eps",
-        #[cfg(system_deps_have_cairo_svg)]
-        "svg",
-    ];
-
-    clap::Command::new("rsvg-convert")
-        .version(concat!("version ", crate_version!()))
-        .about("Convert SVG files to other image formats")
-        .disable_version_flag(true)
-        .disable_help_flag(true)
-        .arg(
-            clap::Arg::new("help")
-                .short('?')
-                .long("help")
-                .help("Display the help")
-                .action(clap::ArgAction::Help)
-        )
-        .arg(
-            clap::Arg::new("version")
-                .short('v')
-                .long("version")
-                .help("Display the version information")
-                .action(clap::ArgAction::Version)
-        )
-        .arg(
-            clap::Arg::new("res_x")
-                .short('d')
-                .long("dpi-x")
-                .num_args(1)
-                .value_name("number")
-                .default_value("96")
-                .value_parser(parse_resolution)
-                .help("Pixels per inch")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("res_y")
-                .short('p')
-                .long("dpi-y")
-                .num_args(1)
-                .value_name("number")
-                .default_value("96")
-                .value_parser(parse_resolution)
-                .help("Pixels per inch")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("zoom_x")
-                .short('x')
-                .long("x-zoom")
-                .num_args(1)
-                .value_name("number")
-                .conflicts_with("zoom")
-                .value_parser(parse_zoom_factor)
-                .help("Horizontal zoom factor")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("zoom_y")
-                .short('y')
-                .long("y-zoom")
-                .num_args(1)
-                .value_name("number")
-                .conflicts_with("zoom")
-                .value_parser(parse_zoom_factor)
-                .help("Vertical zoom factor")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("zoom")
-                .short('z')
-                .long("zoom")
-                .num_args(1)
-                .value_name("number")
-                .value_parser(parse_zoom_factor)
-                .help("Zoom factor")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("size_x")
-                .short('w')
-                .long("width")
-                .num_args(1)
-                .value_name("length")
-                .value_parser(parse_length::<Horizontal, Unsigned>)
-                .help("Width [defaults to the width of the SVG]")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("size_y")
-                .short('h')
-                .long("height")
-                .num_args(1)
-                .value_name("length")
-                .value_parser(parse_length::<Vertical, Unsigned>)
-                .help("Height [defaults to the height of the SVG]")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("top")
-                .long("top")
-                .num_args(1)
-                .value_name("length")
-                .value_parser(parse_length::<Vertical, Signed>)
-                .help("Distance between top edge of page and the image [defaults to 0]")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("left")
-                .long("left")
-                .num_args(1)
-                .value_name("length")
-                .value_parser(parse_length::<Horizontal, Signed>)
-                .help("Distance between left edge of page and the image [defaults to 0]")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("page_width")
-                .long("page-width")
-                .num_args(1)
-                .value_name("length")
-                .value_parser(parse_length::<Horizontal, Unsigned>)
-                .help("Width of output media [defaults to the width of the SVG]")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("page_height")
-                .long("page-height")
-                .num_args(1)
-                .value_name("length")
-                .value_parser(parse_length::<Vertical, Unsigned>)
-                .help("Height of output media [defaults to the height of the SVG]")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("format")
-                .short('f')
-                .long("format")
-                .num_args(1)
-                .value_parser(clap::builder::PossibleValuesParser::new(supported_formats.as_slice()))
-                .ignore_case(true)
-                .default_value("png")
-                .help("Output format")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("output")
-                .short('o')
-                .long("output")
-                .num_args(1)
-                .value_parser(clap::value_parser!(PathBuf))
-                .help("Output filename [defaults to stdout]")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("export_id")
-                .short('i')
-                .long("export-id")
-                .value_parser(clap::builder::NonEmptyStringValueParser::new())
-                .value_name("object id")
-                .help("SVG id of object to export [default is to export all objects]")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("accept-language")
-                .short('l')
-                .long("accept-language")
-                .value_parser(clap::builder::NonEmptyStringValueParser::new())
-                .value_name("languages")
-                .help("Languages to accept, for example \"es-MX,de,en\" [default uses language from the environment]")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("keep_aspect")
-                .short('a')
-                .long("keep-aspect-ratio")
-                .help("Preserve the aspect ratio")
-                .action(clap::ArgAction::SetTrue),
-        )
-        .arg(
-            clap::Arg::new("background")
-                .short('b')
-                .long("background-color")
-                .num_args(1)
-                .value_name("color")
-                .value_parser(clap::builder::NonEmptyStringValueParser::new())
-                .default_value("none")
-                .help("Set the background color using a CSS color spec")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("stylesheet")
-                .short('s')
-                .long("stylesheet")
-            .num_args(1)
-                .value_parser(clap::value_parser!(PathBuf))
-                .value_name("filename.css")
-                .help("Filename of CSS stylesheet to apply")
-                .action(clap::ArgAction::Set),
-        )
-        .arg(
-            clap::Arg::new("unlimited")
-                .short('u')
-                .long("unlimited")
-                .help("Allow huge SVG files")
-                .action(clap::ArgAction::SetTrue),
-        )
-        .arg(
-            clap::Arg::new("keep_image_data")
-                .long("keep-image-data")
-                .help("Keep image data")
-                .conflicts_with("no_keep_image_data")
-                .action(clap::ArgAction::SetTrue),
-        )
-        .arg(
-            clap::Arg::new("no_keep_image_data")
-                .long("no-keep-image-data")
-                .help("Do not keep image data")
-                .conflicts_with("keep_image_data")
-                .action(clap::ArgAction::SetTrue),
-        )
-        .arg(
-            clap::Arg::new("testing")
-                .long("testing")
-                .help("Render images for librsvg's test suite")
-                .hide(true)
-                .action(clap::ArgAction::SetTrue),
-        )
-        .arg(
-            clap::Arg::new("completion")
-                .long("completion")
-                .help("Output shell completion for the given shell")
-                .num_args(1)
-                .action(clap::ArgAction::Set)
-                .value_parser(clap::value_parser!(Shell)),
-        )
-        .arg(
-            clap::Arg::new("FILE")
-                .value_parser(clap::value_parser!(OsString))
-                .help("The input file(s) to convert, you can use - for stdin")
-                .num_args(1..)
-                .action(clap::ArgAction::Append),
-        )
-}
-
 fn print_completions<G: Generator>(gen: G, cmd: &mut clap::Command) {
     clap_complete::generate(gen, cmd, cmd.get_name().to_string(), &mut io::stdout());
 }
 
+extern "C" {
+    fn hb_version_string() -> *const libc::c_char;
+
+    #[cfg(unix)]
+    fn FcGetVersion() -> libc::c_int;
+}
+
+fn harfbuzz_version_string() -> String {
+    unsafe { String::from_glib_none(hb_version_string()) }
+}
+
+#[cfg(unix)]
+fn split_fontconfig_version(version: u32) -> (u32, u32, u32) {
+    let major = version / 10000;
+    let minor = version % 10000 / 100;
+    let revision = version % 100;
+
+    (major, minor, revision)
+}
+
+#[cfg(unix)]
+fn fontconfig_version_string() -> String {
+    let version = unsafe { FcGetVersion() };
+
+    let (major, minor, revision) = split_fontconfig_version(version as u32);
+
+    format!("{major}.{minor}.{revision}")
+}
+
+#[cfg(unix)]
+fn print_fontconfig_version() {
+    let fontconfig_version = fontconfig_version_string();
+    println!("  fontconfig {}", fontconfig_version);
+}
+
+#[cfg(windows)]
+fn print_fontconfig_version() {
+    // fontconfig is not used on Windows, so print nothing
+}
+
+fn print_version() {
+    let cairo_version = cairo::version_string();
+    let pango_version = pango::version_string();
+    let harfbuzz_version = harfbuzz_version_string();
+
+    // We can't get the freetype library version from FT_Library_Version(), because that function
+    // requires a FT_Library.  However, that is hidden as an internal field of PangoFT2FontMap.
+
+    println!("rsvg-convert version {}\n", crate_version!());
+    println!("libraries used:");
+    println!("  cairo {}", cairo_version);
+    println!("  pango {}", pango_version);
+    println!("  harfbuzz {}", harfbuzz_version);
+
+    print_fontconfig_version();
+}
+
 fn parse_args() -> Result<Converter, Error> {
     let cli = build_cli();
     let matches = cli.get_matches();
@@ -1078,6 +876,11 @@ fn parse_args() -> Result<Converter, Error> {
         std::process::exit(0);
     }
 
+    if matches.get_flag("version") {
+        print_version();
+        std::process::exit(0);
+    }
+
     let format_str: &String = matches
         .get_one("format")
         .expect("already provided default_value");
@@ -1210,28 +1013,6 @@ fn parse_args() -> Result<Converter, Error> {
     })
 }
 
-#[derive(Copy, Clone)]
-struct Resolution(f64);
-
-fn parse_resolution(v: &str) -> Result<Resolution, String> {
-    match v.parse::<f64>() {
-        Ok(res) if res > 0.0 => Ok(Resolution(res)),
-        Ok(_) => Err(String::from("Invalid resolution")),
-        Err(e) => Err(format!("{e}")),
-    }
-}
-
-#[derive(Copy, Clone)]
-struct ZoomFactor(f64);
-
-fn parse_zoom_factor(v: &str) -> Result<ZoomFactor, String> {
-    match v.parse::<f64>() {
-        Ok(res) if res > 0.0 => Ok(ZoomFactor(res)),
-        Ok(_) => Err(String::from("Invalid zoom factor")),
-        Err(e) => Err(format!("{e}")),
-    }
-}
-
 fn parse_background_color(s: &str) -> Result<Option<Color>, String> {
     match s {
         "none" | "None" => Ok(None),
@@ -1241,30 +1022,6 @@ fn parse_background_color(s: &str) -> Result<Option<Color>, String> {
     }
 }
 
-fn is_absolute_unit(u: LengthUnit) -> bool {
-    use LengthUnit::*;
-
-    match u {
-        Percent | Em | Ex | Ch => false,
-        Px | In | Cm | Mm | Pt | Pc => true,
-        _ => false,
-    }
-}
-
-fn parse_length<N: Normalize, V: Validate>(s: &str) -> Result<CssLength<N, V>, String> {
-    <CssLength<N, V> as Parse>::parse_str(s)
-        .map_err(|_| format!("Invalid value: The argument '{s}' can not be parsed as a length"))
-        .and_then(|l| {
-            if is_absolute_unit(l.unit) {
-                Ok(l)
-            } else {
-                Err(format!(
-                    "Invalid value '{s}': supported units are px, in, cm, mm, pt, pc"
-                ))
-            }
-        })
-}
-
 fn main() {
     if let Err(e) = parse_args().and_then(|converter| converter.convert()) {
         std::eprintln!("{e}");
@@ -1566,4 +1323,10 @@ mod sizing_tests {
             Size::new(2.0, 10.0),
         );
     }
+
+    #[cfg(unix)]
+    #[test]
+    fn splits_fontconfig_versions() {
+        assert_eq!(split_fontconfig_version(123456), (12, 34, 56));
+    }
 }
