@@ -3,7 +3,6 @@ mod internal_predicates;
 use internal_predicates::file;
 
 use assert_cmd::assert::IntoOutputPredicate;
-use assert_cmd::cargo::cargo_bin_cmd;
 use assert_cmd::Command;
 #[cfg(system_deps_have_cairo_pdf)]
 use chrono::{TimeZone, Utc};
@@ -36,7 +35,7 @@ struct RsvgConvert {}
 impl RsvgConvert {
     #[allow(clippy::new_ret_no_self)]
     fn new() -> Command {
-        cargo_bin_cmd!("rsvg-convert")
+        Command::cargo_bin("rsvg-convert").unwrap()
     }
 
     fn new_with_input<P>(file: P) -> Command
@@ -755,6 +754,16 @@ fn negative_resolution_is_invalid() {
         .stderr(contains("Invalid resolution"));
 }
 
+#[test]
+fn unparsable_resolution_is_invalid() {
+    RsvgConvert::new_with_input("tests/fixtures/dpi.svg")
+        .arg("--dpi-x=blah")
+        .arg("--dpi-y=blah")
+        .assert()
+        .failure()
+        .stderr(contains("invalid value")); // this comes from clap
+}
+
 #[test]
 fn zero_offset_png() {
     RsvgConvert::new_with_input("tests/fixtures/dimensions-in.svg")
@@ -1191,9 +1200,8 @@ fn no_keep_image_data_option() {
     RsvgConvert::accepts_arg("--no-keep-image-data");
 }
 
-fn is_version_output() -> AndPredicate<StartsWithPredicate, TrimPredicate<EndsWithPredicate>, str> {
-    starts_with("rsvg-convert version ")
-        .and(predicates::str::ends_with(env!("CARGO_PKG_VERSION")).trim())
+fn is_version_output() -> RegexPredicate {
+    predicates::str::is_match(r"rsvg-convert version \d+\.\d+\.\d+").unwrap()
 }
 
 #[test]
